@@ -1,5 +1,6 @@
 package CatalystX::SimpleLogin::TraitFor::Controller::Login::WithRedirect;
 use MooseX::MethodAttributes::Role;
+use Moose::Autobox;
 use namespace::autoclean;
 
 requires qw/
@@ -13,15 +14,16 @@ around 'redirect_after_login_uri' => sub {
         return $self->$orig($c, @args);
     }
     return $c->session->{redirect_to_after_login}
-        ? $c->uri_for('/'.$c->session->{redirect_to_after_login})
+        ? $c->session->{redirect_to_after_login}
         : $self->$orig($c, @args);
 };
 
 sub login_redirect {
     my ($self, $c, $message) = @_;
     $c->flash->{error_msg} = $message; # FIXME - Flash horrible
-    $c->session->{redirect_to_after_login} = $c->request->path;
+    $c->session->{redirect_to_after_login} = $c->uri_for($c->action, $c->req->captures, $c->req->args->flatten, $c->req->parameters);
     $c->response->redirect($c->uri_for($self->action_for("login")));
+    $c->detach;
 }
 
 1;
@@ -42,7 +44,7 @@ users who login back to the page they originally requested.
     }
 
     # Turn on in config
-    MyApp->config('Contoller::Login' => { login => 'WithRedirect' });
+    MyApp->config('Contoller::Login' => { traits => 'Login::WithRedirect' });
 
 =head1 DESCRIPTION
 
